\bo.xpl		19-Mar-2020
\BUSTOUT - not to be confused with a game called Breakout 
\
\This game is built with 4x4-pixel blocks.
\Bricks are 4 blocks wide and 1 block high.
\There are 10 bricks per wall.
\The paddle is 7x1 blocks.
\Balls are 1x1 blocks.
\Borders are 1 block thick.
\Thus the width of the playfield, including borders, is (1+10*4+1) blocks.
\The height of the playfield = height of screen = 200/4 blocks.

def	Balls = 5,	\total number of balls for game
	Bricks = 10,	\number of bricks per wall
	Walls = 6,	\number of brick walls
	ScoreMax = (Walls+1) * Walls / 2 * Bricks;	\used to detect win

def	OffX = (320/4 - (1+Bricks*4+1)) /2, \offsets to position field on screen
	OffY = 6;	\offset from top of screen to top wall of bricks, blocks

def	Black, Blue, Green, Cyan, Red, Magenta, Brown, White,  \attribute colors
	Gray, LBlue, LGreen, LCyan, LRed, LMagenta, Yellow, BWhite; \EGA palette

def	BallColor = BWhite,
	BkgndColor = Black;

int	BallsLeft,	\number of balls remaining
	BallX, BallY,	\location of ball on playfield (blocks)
	BreakThru,	\flag: ball hit back border, so bounce back off bricks
	CurDegree,	\current difficulty level (1..5, is 1 during WarmUp)
	Degree,		\difficulty level specified by player (1..5)
	DX, DY,		\direction ball is moving (+/-1)
	OnBoard,	\flag: ball is still in play (still on the playfield)
	PaddleX,	\horizontal paddle position (blocks)
	Score,		\current score
	Slope, SlopeCnt,\determine slope (angle) of ball's trajectory
	WarmUp;		\warm up period starts the ball moving slowly


proc	Check4Exit;		\Esc key exits program
[if KeyHit then
    if ChIn(1) = $1B \Esc\ then
	[SetVid(3);  exit];	\restore normal text mode
];	\Check4Exit


func	Button;			\Return 'true' if any mouse button is pressed
int	M;
[M:= GetMouse;
return (M(2)&7) # 0;
];	\Button


proc	DrawRect(X0, Y0, W, H, Color); \Display a rectangle on playfield
int	X0, Y0,		\coordinates of upper-left corner (blocks)
	W, H,		\width and height (blocks)
	Color;
int	Y;
[X0:= (OffX+X0)*4;  Y0:= Y0*4;	\convert blocks to pixels on playfield
for Y:= Y0 to Y0+H*4-1 do
	[Move(X0, Y);  Line(X0+W*4-1, Y, Color)];
];	\DrawRect


fproc	Shallower;

proc	Steeper;		\Usually make ball trajectory angle more acute
[if Ran(4) = 0 then Shallower
else	[Slope:= Slope/2;
	if Slope < 1 then Slope:= 1;
	];
];	\Steeper


proc	Shallower;		\Usually make ball trajectory angle less acute
[if Ran(4) = 0 then Steeper
else	[Slope:= Slope*2;
	if Slope > 200/8 then Slope:= 200/8;
	];
];	\Shallower


proc	SameSlope;		\Usually leave ball trajectory angle the same
int	RanNum;
[RanNum:= Ran(6);
if RanNum = 0 then Steeper
else if RanNum = 5 then Shallower;
];	\SameSlope


proc	ShowScore;		\Display the Score, times 10
[Attrib(BWhite);
Cursor(33, 10);
IntOut(6, Score);
if Score > 0 then ChOut(6, ^0);
];	\ShowScore


proc	MoveBall;		\Move the ball
int	X, Y, T;		\coordinates (in blocks on playfield), scratch
[DrawRect(BallX, BallY, 1, 1, BkgndColor);	\erase current ball position
X:= BallX;  Y:= BallY+DY;			\try tenative new position
SlopeCnt:= SlopeCnt+1;
if SlopeCnt >= Slope then
	[SlopeCnt:= 0;  X:= X+DX];

while ReadPix((OffX+X)*4, Y*4) # BkgndColor do	\ball hit something
	[if WarmUp > 0 then WarmUp:= WarmUp-1	\speed up after several hits
	else CurDegree:= Degree;
	Sound(1, 1, 3500+Y*40);			\make hit sound
	SlopeCnt:= 0;
	case of
	  Y = 0:				\hit top boundary
		[DY:= -DY;  Y:= Y+DY;  BreakThru:= true];
	  X=0 ! X=1+Bricks*4:			\hit side boundary
		[DX:= -DX;  X:= X+DX];		\bounce horizontally
	  Y = 200/4-2:				\hit paddle
		[if X=PaddleX+0 ! X=PaddleX+1 then [DX:= -1;  Steeper]
		else if X=PaddleX+5 ! X=PaddleX+6 then [DX:= 1;  Steeper]
		else SameSlope;			\Paddle is 0..6 blocks wide
		DY:= -DY;  Y:= Y+DY;		\bounce
		BreakThru:= false;
		]
	other					\hit a brick
		[Score:= Score + Walls - (Y-OffY)/2;
		ShowScore;
		T:= X-1 & ~3;			\left end of brick (in blocks)
		DrawRect(T+1, Y, 4, 1, BkgndColor);	\erase brick
		if DY < 0 then [DY:= -DY;  Y:= Y+DY]	\bounce downward
		else if BreakThru then [DY:= -DY;  Y:= Y+DY]; \bounce
		];
	];
BallX:= X;  BallY:= Y;				\actually move ball
DrawRect(BallX, BallY, 1, 1, BallColor);	\ and show it in new position
if BallY = 200/4 - 1 then OnBoard:= false;	\ball slipped past paddle
];	\MoveBall


proc	MovePaddle;		\Move paddle with mouse
int	M, X;
[M:= GetMouse;
X:= M(0)*(Bricks*4-7)*4/320;	\motion range between playfield borders (pixels)
X:= 1 + X/4;			\convert pixels to blocks on playfield
if X # PaddleX then		\paddle moved
	[DrawRect(PaddleX, 200/4-2, 7, 1, BkgndColor);	\erase old position
	 PaddleX:= X;
	 DrawRect(PaddleX, 200/4-2, 7, 1, LBlue);	\show new position
	];
];	\MovePaddle


proc	LaunchBall;		\Start the ball rolling...
[repeat MovePaddle;  Check4Exit;  until Button;
BallX:= 1 + Ran(Bricks*4);	\pick a random spot in playfield
BallY:= OffY + Walls*2 + 3;
DX:= 1;  DY:= 1;		\move downward with a slight slope
Slope:= 200/8;
SlopeCnt:= 0;
OnBoard:= true;
CurDegree:= 1;  WarmUp:= 5;	\start moving slowly
];	\LaunchBall


proc	SetUp;			\Initialize for start of a game
int	I, J;
[SetVid($13);			\set 320x200x8 graphics
Score:= 0;
ShowScore;
BreakThru:= false;
BallsLeft:= Balls;
for J:= 1 to Balls do		\show all the balls
	DrawRect(1+Bricks*4+1 + 2*J, 14*2, 1, 1, BallColor);
DrawRect(1, 0, Bricks*4, 200/4, BkgndColor);	\draw background
DrawRect(0, 0, 1+Bricks*4+1, 1, BWhite);	\top border
DrawRect(0, 0, 1, 200/4, BWhite);		\left border
DrawRect(1+Bricks*4, 0, 1, 200/4, BWhite);	\right border
for J:= 0 to Walls-1 do				\set up bricks
    [for I:= 0 to Bricks-1 do
	DrawRect(1+I*4, OffY+J*2, 4, 1, J + (if I+J&1 then $70 else $28));
	];
PaddleX:= 1;	\set paddle position where it can be erased and redrawn

Attrib(BkgndColor<<8+Yellow);
Cursor(11, 10);
Text(6, " === BUSTOUT ===");
Cursor(11, 14);
Text(6, "DIFFICULTY (1-5):");
repeat	Degree:= ChIn(1);
	if Degree = $1B \Esc\ then [SetVid(3);  exit];
until Degree>=^1 & Degree<=^5;
ChOut(6, Degree);
Degree:= Degree - ^0;
Cursor(11, 18);
Text(6, "MOUSE MOVES PADDLE");
Cursor(11, 20);
Text(6, "BUTTON STARTS BALL");
repeat MovePaddle;  Check4Exit;  until Button;
DrawRect(1, 10*2, Bricks*4, (20+1-10)*2, BkgndColor);	\erase text
];	\SetUp


int	N;
[loop	[SetUp;
	loop	[LaunchBall;	\and remove it from the balls at the side
		DrawRect(1+Bricks*4+1+2, 14*2, (Balls-(BallsLeft-1))*2, 1, 0);
		while OnBoard do
			[MovePaddle;  MoveBall;
			if Score = ScoreMax then quit;	\we have a winner!
			DelayUS((6-CurDegree)*10000);	\limit ball speed
			Check4Exit;
			];
		Sound(1, 18, 5000);
		DrawRect(BallX, BallY, 1, 1, BkgndColor); \remove ball
		BallsLeft:= BallsLeft-1;
		if BallsLeft = 0 then quit;		\sorry, you lose
		];
	if Score = ScoreMax then 
		[for N:= 1 to BallsLeft do		\bonus points for
			[Score:= Score + 50;		\ remaining balls
			ShowScore;
			DrawRect(1+Bricks*4+1+2, 14*2,	\remove balls
				(Balls-(BallsLeft-N))*2, 1, 0);
			Sound(0, 2, 1);  Sound(1, 2, 1500);
			Sound(0, 2, 1);  Sound(1, 2, 1000);
			];
		Attrib(BkgndColor<<8+Yellow);
		Cursor(18, 12);
		Text(6, "WOWEE!");
		]
	else   [Attrib(BkgndColor<<8+Yellow);
		Cursor(18, 12);
		Text(6, "GAME");
		Cursor(18, 14);
		Text(6, "OVER");
	       ];
	repeat MovePaddle;  Check4Exit;  until Button;
	];
]
