\VMouse.xpl	19-Mar-2020
\Example of a mouse pointer for VESA graphics
\Compile with 32-bit XPLPX (xpx.bat).
\
\Revisions:
\27-Apr-2007: Released.
\ 5-May-2007: Eliminate flicker when mouse button is pressed.
\ 1-Feb-2014: Initialize MouseX,Y. More descriptive names (e.g: MX to MouseX).
\19-Mar-2020: Greatly simplified by using built-in mouse intrinsics.

def	VMode=$112,		\24-bit graphic video mode
	ScrW=640, ScrH=480;	\screen dimensions (pixels)

int	C0, C1, C2, C3,		\corner colors for gradient
	RX0, RY0, RX1, RY1,	\rectangle corners (pixels)
	MButton;		\bool: mouse button went from up to down



proc	Exit(Msg);		\Display (error) message and terminate program
char	Msg;
[SetVid(3);			\restore normal text mode
Text(0, Msg);
CrLf(0);
exit;
];	\Exit



proc	ShowGradientRectangle(X, Y, W, H);	\Display a gradient rectangle
int	X, Y, W, H;	\upper left corner coordinates, width, height (pixels)
int	I, J, C, CX0, CX1;
[if W < 0 then [W:= -W;  X:= X-W];	\handle backward arguments
 if H < 0 then [H:= -H;  Y:= Y-H];
for J:= Y, Y+H-1 do			\for each scan line...
	[CX0:= (C1-C0)*J/(H*H);		\interpolate (sort of) colors from top
	 CX1:= (C3-C2)*J/(H*H);		\ to bottom of rectangle
	 for I:= X, X+W-1 do		\for each pixel...
		[C:= (CX1-CX0)*I/(W*W);	\interpolate from left to right
		Point(I, J, C);
		];
	];
];	\ShowGradientRectangle



proc	DragRect;		\Use mouse to drag a colorful rectangle
int	M, Button;
int	MouseX, MouseY;		\current mouse coordinates (pixels)
[MoveMouse;			\move pointer to track actual mouse position

M:= GetMouse;			\get address of array of mouse state information
MouseX:= M(0);
MouseY:= M(1);
Button:= M(2) & 1;		\use left button

if Button then
	[if ~MButton then	\mouse button just went down
		[MButton:= true;
		RX0:= MouseX;  RY0:= MouseY;	\establish corner of rectangle
		C0:= Ran(256);			\ along with its colors
		C1:= Ran(256) << 8;
		C2:= Ran(256) << 16;
		C3:= 0;
		];

	\Mouse button is held down
	if MouseX#RX1 ! MouseY#RY1 then		\mouse moved
		[RX1:= MouseX;  RY1:= MouseY;	\second corner of rectangle
		ShowMouse(false);		\turn off pointer while drawing
		ShowGradientRectangle(RX0, RY0, RX1-RX0, RY1-RY0); \ to screen
		ShowMouse(true);
		];
	]
else	MButton:= false;			\mouse button is released
];	\DragRect



[SetVid(VMode);

if not OpenMouse then Exit("This program requires a mouse.");

Move((ScrW-18*8)/2, ScrH/2);			\center message on screen
Text($106, "Drag me beautiful!");

ShowMouse(true);				\display mouse pointer
repeat DragRect until KeyHit;

SetVid(3);					\restore normal text mode
]
